const fs = require('fs').promises;
const path = require('path');

const staticSrcPath = path.join(__dirname, '.next/static');
const staticDestPath = path.join(__dirname, '.next/standalone/apps/adminpanel/.next/static');

const publicSrcPath = path.join(__dirname, 'public');
const publicDestPath = path.join(__dirname, '.next/standalone/apps/adminpanel/public');

function copyAssets(src, dest) {
  return fs.mkdir(dest, { recursive: true })
    .then(() => fs.readdir(src, { withFileTypes: true }))
    .then(items => {
      const promises = items.map(item => {
        const srcPath = path.join(src, item.name);
        const destPath = path.join(dest, item.name);

        if (item.isDirectory()) {
          return copyAssets(srcPath, destPath);
        } else {
          return fs.copyFile(srcPath, destPath);
        }
      });
      return Promise.all(promises);
    })
    .catch(err => {
      console.error(`Error: ${err}`);
      throw err;
    });
}

function waitForPathToExistsOrTimeout(path, timeout) {
  return new Promise((resolve, reject) => {
    const interval = 100;
    const timeoutId = setTimeout(() => {
      clearInterval(intervalId);
      reject(`Timeout waiting for path: ${path}`);
    }, timeout);
    const intervalId = setInterval(() => {
      fs.access(path)
        .then(() => {
          clearInterval(intervalId);
          clearTimeout(timeoutId);
          resolve();
        })
        .catch(() => { });
    }, interval);
  });
}



const greenTick = `\x1b[32m\u2713\x1b[0m`;
const redCross = `\x1b[31m\u274C\x1b[0m`;
waitForPathToExistsOrTimeout('.next/standalone', 5000).then(() => {
  //wait 1 second to prevent race condition on windows
  setTimeout(() => {
    copyAssets(staticSrcPath, staticDestPath)
      .then(() => copyAssets(publicSrcPath, publicDestPath))
      .then(() => console.log(`${greenTick} Assets copied successfully`))
      .catch(err => console.error(`${redCross} Failed to copy assets: ${err}`));
  }, 1000);
}).catch(err => console.error(`${redCross} Failed to copy assets: ${err}`));
