
import WebSocket, { Server } from 'ws';
import net from 'net';
import aedes from 'aedes';
import { getLogger } from 'protobase';

const logger = getLogger()

export const startMqtt = (config, mqttPort=1883, webSocketPort=3003) => {
    const aedesInstance = new aedes();
    aedesInstance.authenticate = function (client, username, password, callback) {
        if (!username) {
            logger.trace({}, "MQTT anonymous login request")
        } else {
            logger.trace({ username }, "MQTT user login request: " + username)
        }

        if (config.mqtt.auth) {
            if (!username || !password) {
                logger.error({}, "MQTT anonymous login refused: mqtt requires auth")
            } else {
                //TODO: auth
                callback(null, true)
            }
        } else {
            callback(null, true)
        }
    }
    
    const wss = new Server({
        path: '/websocket',
        port: webSocketPort
    });

    wss.on('connection', (ws: WebSocket) => {
        const stream = WebSocket.createWebSocketStream(ws, { decodeStrings: false });
        aedesInstance.handle(stream as any);
    });

    const mqttServer = net.createServer((socket) => {
        aedesInstance.handle(socket);
    });

    mqttServer.listen(mqttPort, () => {
        logger.debug({ service: { protocol: "mqtt", port: mqttPort } }, "Service started: MQTT")
    });
}