import * as Zod from 'zod';
interface ZodExtensions {
    indexed(indexFn?: Function): this;
    groupIndex(groupName?: string, groupCode?: string): this;
    linkTo(getElements: Function, getId: Function, readIds: Function, displayKey?: string | Function, options?: {
        deleteOnCascade: boolean;
    }): this;
    label(caption: string): this;
    hint(hintText: string): this;
    synthesize(fn: Function): this;
    display(views?: string[] | undefined): this;
    columnWidth(width: number): this;
    hidden(views?: Array<"list" | "sheet" | "preview" | "add" | "edit">): this;
    color(): this;
    textArea(height?: number): this;
    file({ initialPath, extensions }: {
        initialPath?: string;
        extensions?: string[];
    }): this;
    generate(val: any, force?: boolean): this;
    before(field: string): this;
    after(field: string): this;
    datePicker(type?: 'single' | 'month' | 'year'): this;
    dependsOn(field: string, value?: any): this;
    location(latKey: string, lonKey: string, locationType?: "point" | "boxarea", locationOptions?: any, locationStyle?: any): this;
    generateOptions(call: Function): this;
    visible(visibilityCheck: Function): this;
    sequence(): this;
    choices(): this;
    secret(): this;
    static(): this;
    id(): this;
    search(searchable?: boolean): this;
    filter(display?: boolean): this;
    displayOptions(options: any): this;
    size(size: number): this;
    group(group: number): this;
    name(key: string): this;
    help(description: string): this;
    defaultValue(value: any): this;
    onList(eventHandler: string, eventContext?: 'client' | 'server' | undefined, eventParams?: any): this;
    onCreate(eventHandler: string, eventContext?: 'client' | 'server' | undefined, eventParams?: any): this;
    onRead(eventHandler: string, eventContext?: 'client' | 'server' | undefined, eventParams?: any): this;
    onUpdate(eventHandler: string, eventContext?: 'client' | 'server' | undefined, eventParams?: any): this;
    onDelete(eventHandler: string, eventContext?: 'client' | 'server' | undefined, eventParams?: any): this;
    on(eventName: string, eventHandler: string, eventContext?: 'client' | 'server' | undefined, eventParams?: any): this;
}
declare module 'zod' {
    interface ZodString extends ZodExtensions {
    }
    interface ZodNumber extends ZodExtensions {
    }
    interface ZodBoolean extends ZodExtensions {
    }
    interface ZodAny extends ZodExtensions {
    }
    interface ZodOptional extends ZodExtensions {
    }
    interface ZodArray extends ZodExtensions {
    }
    interface ZodUnion extends ZodExtensions {
    }
    interface ZodObject extends ZodExtensions {
    }
    interface ZodRecord extends ZodExtensions {
    }
    interface ZodDate extends ZodExtensions {
    }
}
export declare const Schema: typeof Zod.z;
export * from 'zod';
export declare function initSchemaSystem(): void;
export declare const BaseSchema: Zod.ZodObject<{
    id: Zod.ZodString;
}, "strip", Zod.ZodTypeAny, {
    id?: string;
}, {
    id?: string;
}>;
