"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getApp = void 0;
const express_1 = __importDefault(require("express"));
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const cors_1 = __importDefault(require("cors"));
const pino_http_1 = __importDefault(require("pino-http"));
const protobase_1 = require("protobase");
const handler_1 = require("./handler");
const express_list_endpoints_1 = __importDefault(require("express-list-endpoints"));
let app;
const getApp = (fn = (app) => { }) => {
    if (!app) {
        const logger = (0, protobase_1.getLogger)();
        const config = (0, protobase_1.getConfig)();
        app = (0, express_1.default)();
        fn(app);
        app.set('json spaces', 2);
        app.use((0, cors_1.default)());
        app.use((0, cookie_parser_1.default)());
        app.use(express_1.default.json({ limit: '50mb' }));
        app.use((0, pino_http_1.default)({
            customSuccessMessage: function (req, res) {
                return `${req.method} ${req.originalUrl} - ${res.statusCode}`;
            },
            serializers: {
                req: (req) => {
                    if (process.env.NODE_ENV === "development") {
                        return {
                            method: req.method,
                            url: req.url,
                        };
                    }
                    else {
                        return req;
                    }
                },
                res: (res) => {
                    if (process.env.NODE_ENV === "development") {
                        return {
                            code: res.statusCode,
                        };
                    }
                    else {
                        return res;
                    }
                },
            },
            ...config.logger,
            useLevel: 'trace'
        }));
        app.get(global.defaultRoute + '/endpoints', (0, handler_1.handler)(async (req, res, session, next) => {
            (0, protobase_1.hasPermission)(session, 'admin/endpoints/list');
            res.send((0, express_list_endpoints_1.default)(app));
        }));
        logger.debug({ route: global.defaultRoute }, "Default route: " + global.defaultRoute);
    }
    return app;
};
exports.getApp = getApp;
