"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeFeature = exports.addFeature = exports.hasFeature = exports.removeFileWithImports = exports.removeObjectLiteralProperty = exports.addObjectLiteralProperty = exports.addElementToArrayLiteral = exports.getPropertyValueFromObjectLiteral = exports.removeImportFromSourceFile = exports.addImportToSourceFile = exports.ImportType = exports.extractChainCalls = exports.toSourceFile = exports.getSourceFile = exports.getDefinition = exports.getDefinitions = exports.getImport = void 0;
const ts_morph_1 = require("ts-morph");
const fspath = __importStar(require("path"));
const getRoot_1 = require("./getRoot");
const protobase_1 = require("protobase");
const logger = (0, protobase_1.getLogger)();
const getImport = (sourceFile, identifier) => {
    const importDeclarations = sourceFile.getImportDeclarations();
    for (const importDeclaration of importDeclarations) {
        const namedImports = importDeclaration.getNamedImports();
        for (const namedImport of namedImports) {
            if (namedImport.getName() === identifier) {
                return importDeclaration.getModuleSpecifierValue();
            }
        }
        const defaultImport = importDeclaration.getDefaultImport();
        if (defaultImport && defaultImport.getText() === identifier) {
            return importDeclaration.getModuleSpecifierValue();
        }
    }
};
exports.getImport = getImport;
const getDefinitions = (sourceFile, def, numParam = 1) => {
    const callExpressions = sourceFile.getDescendantsOfKind(ts_morph_1.SyntaxKind.CallExpression);
    const callsToDef = callExpressions.filter(callExpr => {
        const args = callExpr.getArguments();
        const expression = callExpr.getExpression();
        return expression.getKind() === ts_morph_1.SyntaxKind.Identifier && expression.getText() === 'Protofy' && args.length && args[0].getText() == def;
    }).map(x => x.getArguments()[numParam]);
    return callsToDef;
};
exports.getDefinitions = getDefinitions;
const getDefinition = (sourceFile, def, numParam = 1) => {
    const callExpressions = sourceFile.getDescendantsOfKind(ts_morph_1.SyntaxKind.CallExpression);
    const callToDef = callExpressions.find(callExpr => {
        const args = callExpr.getArguments();
        const expression = callExpr.getExpression();
        return expression.getKind() === ts_morph_1.SyntaxKind.Identifier && expression.getText() === 'Protofy' && args.length && args[0].getText() == def;
    });
    return callToDef ? callToDef.getArguments()[numParam] : undefined;
};
exports.getDefinition = getDefinition;
const getSourceFile = (path) => {
    const project = new ts_morph_1.Project();
    const sourceFile = project.addSourceFileAtPath(path);
    return sourceFile;
};
exports.getSourceFile = getSourceFile;
const toSourceFile = (code) => {
    const project = new ts_morph_1.Project({ useInMemoryFileSystem: true });
    let source = project.createSourceFile('_temp1.tsx', code, { overwrite: true });
    return source;
};
exports.toSourceFile = toSourceFile;
const extractChainCalls = (callExpr) => {
    const calls = [];
    let currentExpression = callExpr;
    while (currentExpression && currentExpression.getKind() === ts_morph_1.SyntaxKind.CallExpression) {
        const signature = {
            name: currentExpression.getExpression().getLastChild().getText(),
            params: currentExpression.getArguments().map(arg => arg.getText())
        };
        calls.unshift(signature);
        currentExpression = currentExpression.getExpression().getFirstChildIfKind(ts_morph_1.SyntaxKind.CallExpression);
    }
    return calls;
};
exports.extractChainCalls = extractChainCalls;
var ImportType;
(function (ImportType) {
    ImportType[ImportType["DEFAULT"] = 0] = "DEFAULT";
    ImportType[ImportType["NAMED"] = 1] = "NAMED";
})(ImportType || (exports.ImportType = ImportType = {}));
const addImportToSourceFile = (sourceFile, key, type, path) => {
    if ((0, exports.getImport)(sourceFile, key)) {
        logger.warn(`"${key}" is already imported`);
        return;
    }
    switch (type) {
        case ImportType.DEFAULT:
            sourceFile.addImportDeclaration({
                defaultImport: key,
                moduleSpecifier: path
            });
            break;
        case ImportType.NAMED:
            sourceFile.addImportDeclaration({
                namedImports: [key],
                moduleSpecifier: path
            });
            break;
    }
};
exports.addImportToSourceFile = addImportToSourceFile;
const removeImportFromSourceFile = (sourceFile, path) => {
    const importDeclarations = sourceFile.getImportDeclarations();
    for (const importDeclaration of importDeclarations) {
        if (importDeclaration.getModuleSpecifierValue() === path) {
            importDeclaration.remove();
            break;
        }
    }
};
exports.removeImportFromSourceFile = removeImportFromSourceFile;
const getPropertyValueFromObjectLiteral = (objectLiteral, key) => {
    try {
        const property = objectLiteral.getProperty(key);
        if (!property || property.getKind() !== ts_morph_1.SyntaxKind.PropertyAssignment)
            throw "Property doesn't exist";
        const propertyAssignment = property.asKindOrThrow(ts_morph_1.SyntaxKind.PropertyAssignment);
        if (!propertyAssignment || !propertyAssignment.getInitializer)
            throw "Property doesn't exist";
        const initializer = propertyAssignment.getInitializer();
        if (!initializer)
            throw "Property doesn't exist";
        return initializer;
    }
    catch (e) {
        console.log("Error retrieving property. Doesn't exist");
    }
};
exports.getPropertyValueFromObjectLiteral = getPropertyValueFromObjectLiteral;
const addElementToArrayLiteral = (arrayLiteral, elem) => {
    try {
        if (!arrayLiteral || arrayLiteral.getKind() !== ts_morph_1.SyntaxKind.ArrayLiteralExpression)
            throw "Element is not an array";
        arrayLiteral.addElement(elem);
    }
    catch (e) {
        console.log("Error adding element to array literal");
    }
};
exports.addElementToArrayLiteral = addElementToArrayLiteral;
const addObjectLiteralProperty = (objectLiteral, key, value) => {
    const existingProperty = objectLiteral.getProperty(p => p instanceof ts_morph_1.PropertyAssignment &&
        ((p.getNameNode().getKind() === ts_morph_1.SyntaxKind.Identifier && p.getName() === key) ||
            (p.getNameNode().getKind() === ts_morph_1.SyntaxKind.ComputedPropertyName &&
                p.getNameNode().getText() === `["${key}"]`)));
    if (existingProperty) {
        logger.warn(`Object already has key: "${key}".`);
        return;
    }
    const isNormalIdentifier = /^[a-zA-Z_$][a-zA-Z\d_$]*$/.test(key);
    const propertyName = isNormalIdentifier ? key : `["${key}"]`;
    objectLiteral.addPropertyAssignment({
        name: propertyName,
        initializer: value,
    });
};
exports.addObjectLiteralProperty = addObjectLiteralProperty;
const removeObjectLiteralProperty = (objectLiteral, key) => {
    const existingProperty = objectLiteral.getProperty(p => p instanceof ts_morph_1.PropertyAssignment &&
        ((p.getNameNode().getKind() === ts_morph_1.SyntaxKind.Identifier && p.getName() === key) ||
            (p.getNameNode().getKind() === ts_morph_1.SyntaxKind.ComputedPropertyName &&
                p.getNameNode().getText() === `["${key}"]`)));
    if (existingProperty) {
        existingProperty.remove();
    }
    else {
        logger.warn(`Key "${key}" does not exist on the object.`);
    }
};
exports.removeObjectLiteralProperty = removeObjectLiteralProperty;
const removeFileWithImports = async (root, value, type, indexFilePath, req, fs) => {
    const name = value.name;
    const localPath = './' + fspath.basename(name);
    const sourceFile = (0, exports.getSourceFile)(fspath.join(root, indexFilePath));
    const arg = (0, exports.getDefinition)(sourceFile, type);
    if (!arg) {
        throw "No link definition schema marker found for file: " + localPath;
    }
    (0, exports.removeObjectLiteralProperty)(arg, name);
    (0, exports.removeImportFromSourceFile)(sourceFile, localPath);
    sourceFile.saveSync();
    const filePath = (0, getRoot_1.getRoot)(req) + 'packages/app/' + type.replace(/"/g, '') + '/' + fspath.basename(value.name) + '.ts';
    try {
        await fs.unlink(filePath);
    }
    catch (error) {
        logger.error({ error, filePath }, "Error deleting file");
    }
};
exports.removeFileWithImports = removeFileWithImports;
//features functions
const hasFeature = (sourceFile, key) => {
    let arg = (0, exports.getDefinition)(sourceFile, '"features"');
    if (arg) {
        const property = arg.getProperty(key);
        return property !== undefined;
    }
    else {
        logger.info("Cannot check for feature due to missing features marker");
        return false;
    }
};
exports.hasFeature = hasFeature;
const addFeature = async (sourceFile, key, value) => {
    if ((0, exports.hasFeature)(sourceFile, key)) {
        logger.info({ feature: key }, "Feature already exists, not adding");
        return;
    }
    logger.debug('Marker found, writing object');
    let arg = (0, exports.getDefinition)(sourceFile, '"features"');
    arg.addPropertyAssignment({
        name: key,
        initializer: value
    });
    await sourceFile.save();
};
exports.addFeature = addFeature;
const removeFeature = async (sourceFile, key) => {
    if (!(0, exports.hasFeature)(sourceFile, key)) {
        logger.error({ feature: key }, "Feature not found, cannot remove");
        return;
    }
    logger.debug('Marker found, removing object property');
    let arg = (0, exports.getDefinition)(sourceFile, '"features"');
    const propertyToRemove = arg.getProperty(key);
    propertyToRemove.remove();
    await sourceFile.save();
};
exports.removeFeature = removeFeature;
