"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDBOptions = exports.dbProvider = exports.closeDBS = exports.getDB = exports.connectDB = void 0;
const sqlite_1 = require("./dbproviders/sqlite");
const getDBPath = (dbPath) => {
    const [dbName, env] = dbPath.split('/').reverse();
    return '../../data/' + (env ? env + '/databases/' : 'databases/') + dbName;
};
const connectDB = (dbPath, initialData, options) => {
    return sqlite_1.ProtoSqliteDB.initDB(getDBPath(dbPath), initialData, options);
};
exports.connectDB = connectDB;
const getDB = (dbPath, req, session) => {
    return sqlite_1.ProtoSqliteDB.connect(getDBPath(dbPath));
};
exports.getDB = getDB;
const closeDBS = async () => {
    return await sqlite_1.ProtoSqliteDB.closeDBS();
};
exports.closeDBS = closeDBS;
exports.dbProvider = {
    connectDB: exports.connectDB,
    getDB: exports.getDB,
    closeDBS: exports.closeDBS
};
const getDBOptions = (modelType, dbOptions) => {
    return {
        indexes: modelType.getIndexes(),
        groupIndexes: modelType.getGroupIndexes(),
        dbOptions: {
            batch: false,
            batchLimit: 100,
            batchTimeout: 2000,
            ...dbOptions
        }
    };
};
exports.getDBOptions = getDBOptions;
