"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.requireAdmin = exports.handler = exports.getAuth = void 0;
const protobase_1 = require("protobase");
const crypt_1 = require("./crypt");
const logger = (0, protobase_1.getLogger)();
const getAuth = (req) => {
    //try to recover identify from token
    let decoded;
    let session;
    try {
        session = JSON.parse(req.cookies.session);
    }
    catch (e) {
        session = null;
    }
    var token = '';
    if (req.query.token || (session && session.token)) {
        token = req.query.token ? req.query.token : session.token;
        if (Array.isArray(token)) {
            token = token[0];
        }
        try {
            decoded = (0, protobase_1.createSession)((0, crypt_1.verifyToken)(token));
        }
        catch (error) {
            logger.error({ error }, "Error reading token");
            decoded = (0, protobase_1.createSession)();
        }
    }
    else {
        decoded = (0, protobase_1.createSession)();
    }
    return { session: decoded, token };
};
exports.getAuth = getAuth;
const handler = fn => async (req, res, next) => {
    try {
        const params = (0, exports.getAuth)(req);
        await fn(req, res, { ...params.session, token: params.token }, next);
    }
    catch (e) {
        // logger.error({error: e}, "Error processing api call")
        if (e instanceof protobase_1.ZodError) {
            const err = e.flatten();
            res.status(500).send(err);
        }
        else if (e.toString() == 'E_PERM') {
            res.status(403).send({ error: e.toString() });
        }
        else if (e.toString() == 'E_AUTH') {
            res.status(401).send({ error: e.toString() });
        }
        else if (e.toString() == 'File already exists') {
            res.status(409).send({ error: e.toString() });
        }
        else {
            res.status(500).send({ error: e.toString() });
        }
    }
};
exports.handler = handler;
const requireAdmin = () => (0, exports.handler)(async (req, res, session, next) => {
    if (!session || !session.user.admin) {
        res.status(401).send({ error: "Unauthorized" });
        return;
    }
    next();
});
exports.requireAdmin = requireAdmin;
