"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProtoDB = void 0;
const protobase_1 = require("protobase");
const logger = (0, protobase_1.getLogger)();
const dbHandlers = {};
class ProtoDB {
    constructor(location, options, config) {
        this.location = location;
        this.options = options;
        this.config = config !== null && config !== void 0 ? config : {};
        this.capabilities = [];
        if (config && config.context && config.name) {
            config.context[config.name] = this;
        }
    }
    get status() {
        return "open";
    }
    on(event, cb) {
        if (event == 'open') {
            cb();
        }
    }
    hasCapability(capability) {
        return this.capabilities.includes(capability);
    }
    async close() { }
    async exists(key) {
        try {
            const result = this.get(key);
            return result ? true : false;
        }
        catch (e) {
            return false;
        }
    }
    static getInstance(location, options, config) {
        throw "Error: derived classes of ProtoDB should implement getInstance";
    }
    static connect(location, options, config) {
        if (!(location in dbHandlers)) {
            //@ts-ignore
            dbHandlers[location] = this.getInstance(location, options, config);
            process.on('SIGINT', async () => {
                logger.info('Closing database and terminating process...');
                //@ts-ignore
                await dbHandlers[location].close();
                process.exit(0);
            });
        }
        return dbHandlers[location];
    }
    static async closeDBS() {
        const keys = Object.keys(dbHandlers);
        for (const key of keys) {
            await dbHandlers[key].close();
        }
    }
    static async initDB(dbPath, initialData = {}, options) {
        throw new Error('Static method initDB must be implemented');
    }
}
exports.ProtoDB = ProtoDB;
